@@ -4,7 +4,7 @@
   FILE: icalvalue.c
   CREATOR: eric 02 May 1999
   
-  $Id: icalvalue.c 630 2007-12-12 17:44:56Z msuman $
+  $Id: icalvalue.c 633 2008-02-18 07:26:45Z pchen $
 
 
  (C) COPYRIGHT 2000, Eric Busboom, http://www.softwarestudio.org
@@ -747,7 +747,7 @@ static char* icalvalue_binary_as_ical_string(const icalvalue* value) {
 
     data = icalvalue_get_binary(value);
 
-    str = (char*)icalmemory_tmp_buffer(60);
+    str = (char*)icalmemory_new_buffer(60);
     sprintf(str,"icalvalue_binary_as_ical_string is not implemented yet");
 
     return str;
@@ -758,7 +758,7 @@ static char* icalvalue_binary_as_ical_string(const icalvalue* value) {
     
 static char* icalvalue_int_as_ical_string(const icalvalue* value) {
     int data;
-    char* str = (char*)icalmemory_tmp_buffer(MAX_INT_DIGITS); 
+    char* str = (char*)icalmemory_new_buffer(MAX_INT_DIGITS); 
 
     icalerror_check_arg_rz( (value!=0),"value");
 
@@ -773,7 +773,7 @@ static char* icalvalue_utcoffset_as_ical_string(const icalvalue* value)
 {    
     int data,h,m,s;
     char sign;
-    char* str = (char*)icalmemory_tmp_buffer(9);
+    char* str = (char*)icalmemory_new_buffer(9);
 
     icalerror_check_arg_rz( (value!=0),"value");
 
@@ -804,7 +804,7 @@ static char* icalvalue_string_as_ical_string(const icalvalue* value) {
     icalerror_check_arg_rz( (value!=0),"value");
     data = value->data.v_string;
 
-    str = (char*)icalmemory_tmp_buffer(strlen(data)+1);   
+    str = (char*)icalmemory_new_buffer(strlen(data)+1);   
 
     strcpy(str,data);
 
@@ -828,7 +828,6 @@ static char* icalvalue_recur_as_ical_string(const icalvalue* value)
 static char* icalvalue_text_as_ical_string(const icalvalue* value) {
     char *str;
     char *str_p;
-    char *rtrn;
     const char *p;
     size_t buf_sz;
 
@@ -887,11 +886,7 @@ static char* icalvalue_text_as_ical_string(const icalvalue* value) {
 
     icalmemory_append_char(&str,&str_p,&buf_sz,'\0');
 
-    rtrn = icalmemory_tmp_copy(str);
-
-    icalmemory_free_buffer(str);
-
-    return rtrn;
+    return str;
 }
 
 
@@ -909,10 +904,10 @@ icalvalue_attach_as_ical_string(const icalvalue* value)
 	const char *url;
 
 	url = icalattach_get_url (a);
-	str = icalmemory_tmp_buffer (strlen (url) + 1);
+	str = icalmemory_new_buffer (strlen (url) + 1);
 	strcpy (str, url);
 	return str;
-    } else
+    } else 
 	return icalvalue_binary_as_ical_string (value);
 }
 
@@ -957,7 +952,7 @@ static char* icalvalue_date_as_ical_string(const icalvalue* value) {
     icalerror_check_arg_rz( (value!=0),"value");
     data = icalvalue_get_date(value);
 
-    str = (char*)icalmemory_tmp_buffer(9);
+    str = (char*)icalmemory_new_buffer(9);
  
     str[0] = 0;
     print_date_to_string(str,&data);
@@ -972,7 +967,7 @@ void print_datetime_to_string(char* str,  const struct icaltimetype *data)
     print_time_to_string(str,data);
 }
 
-static const char* icalvalue_datetime_as_ical_string(const icalvalue* value) {
+static char* icalvalue_datetime_as_ical_string(const icalvalue* value) {
     
     struct icaltimetype data;
     char* str;
@@ -989,7 +984,7 @@ static const char* icalvalue_datetime_as_ical_string(const icalvalue* value) {
 
     data = icalvalue_get_datetime(value);
 
-    str = (char*)icalmemory_tmp_buffer(20);
+    str = (char*)icalmemory_new_buffer(20);
  
     str[0] = 0;
 
@@ -1006,7 +1001,7 @@ static char* icalvalue_float_as_ical_string(const icalvalue* value) {
     icalerror_check_arg_rz( (value!=0),"value");
     data = icalvalue_get_float(value);
 
-    str = (char*)icalmemory_tmp_buffer(15);
+    str = (char*)icalmemory_new_buffer(15);
 
     sprintf(str,"%f",data);
 
@@ -1021,14 +1016,14 @@ static char* icalvalue_geo_as_ical_string(const icalvalue* value) {
 
     data = icalvalue_get_geo(value);
 
-    str = (char*)icalmemory_tmp_buffer(25);
+    str = (char*)icalmemory_new_buffer(25);
 
     sprintf(str,"%f;%f",data.lat,data.lon);
 
     return str;
 }
 
-static const char* icalvalue_datetimeperiod_as_ical_string(const icalvalue* value) {
+static char* icalvalue_datetimeperiod_as_ical_string(const icalvalue* value) {
     struct icaldatetimeperiodtype dtp = icalvalue_get_datetimeperiod(value);
 
     icalerror_check_arg_rz( (value!=0),"value");
@@ -1040,7 +1035,7 @@ static const char* icalvalue_datetimeperiod_as_ical_string(const icalvalue* valu
     }
 }
 
-static const char* icalvalue_period_as_ical_string(const icalvalue* value) {
+static char* icalvalue_period_as_ical_string(const icalvalue* value) {
     struct icalperiodtype data;
     icalerror_check_arg_rz( (value!=0),"value");
     data = icalvalue_get_period(value);
@@ -1049,7 +1044,7 @@ static const char* icalvalue_period_as_ical_string(const icalvalue* value) {
 
 }
 
-static const char* icalvalue_trigger_as_ical_string(const icalvalue* value) {
+static char* icalvalue_trigger_as_ical_string(const icalvalue* value) {
 
     struct icaltriggertype data;
 
@@ -1064,7 +1059,7 @@ static const char* icalvalue_trigger_as_ical_string(const icalvalue* value) {
 
 }
 
-const char*
+char*
 icalvalue_as_ical_string(const icalvalue* value)
 {
     if(value == 0){
@@ -1133,14 +1128,14 @@ icalvalue_as_ical_string(const icalvalue* value)
     case ICAL_TRANSP_VALUE:
     case ICAL_CLASS_VALUE:
         if(value->x_value !=0){
-            return icalmemory_tmp_copy(value->x_value);
+            return icalmemory_strdup (value->x_value);
         }
 
-        return icalproperty_enum_to_string(value->data.v_enum);
+        return icalmemory_strdup (icalproperty_enum_to_string(value->data.v_enum));
         
     case ICAL_X_VALUE: 
 	if (value->x_value != 0)
-            return icalmemory_tmp_copy(value->x_value);
+            return icalmemory_strdup (value->x_value);
 
     /* FALLTHRU */
 
@@ -1294,9 +1289,14 @@ icalvalue_compare(const icalvalue* a, const icalvalue *b)
 	case ICAL_RECUR_VALUE:
 	{
 	    int r;
+	    char *temp1, *temp2;
 
-	    r =  strcmp(icalvalue_as_ical_string(a),
-			  icalvalue_as_ical_string(b));
+	    temp1 = icalvalue_as_ical_string (a);
+	    temp2 = icalvalue_as_ical_string (b);
+	    r =  strcmp(temp1,
+			  temp2);
+	    free (temp1);
+	    free (temp2);
 
 	    if (r > 0) { 	
 		return ICAL_XLICCOMPARETYPE_GREATER;
@@ -1413,6 +1413,7 @@ int icalvalue_encode_ical_string(const char *szText, char *szEncText, int nMaxBu
         }
 
     strcpy(szEncText, ptr);
+    free (ptr);
 
     icalvalue_free ((icalvalue*)value);
 
