@@ -4,7 +4,7 @@
  * Authors : Chris Toshok <toshok@ximian.com> 
  *           Michael Zucchi <notzed@ximian.com>
  *
- * Copyright (C) 2001-2003 Ximian, Inc. (www.ximian.com)
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of version 2 of the GNU Lesser General Public 
@@ -105,7 +105,7 @@ static void
 nntp_folder_sync_online (CamelFolder *folder, CamelException *ex)
 {
 	CAMEL_SERVICE_REC_LOCK(folder->parent_store, connect_lock);
-	camel_folder_summary_save (folder->summary);
+	camel_folder_summary_save_to_db (folder->summary, ex);
 	CAMEL_SERVICE_REC_UNLOCK(folder->parent_store, connect_lock);
 }
 
@@ -113,7 +113,7 @@ static void
 nntp_folder_sync_offline (CamelFolder *folder, CamelException *ex)
 {
 	CAMEL_SERVICE_REC_LOCK(folder->parent_store, connect_lock);
-	camel_folder_summary_save (folder->summary);
+	camel_folder_summary_save_to_db (folder->summary, ex);
 	CAMEL_SERVICE_REC_UNLOCK(folder->parent_store, connect_lock);
 }
 
@@ -274,6 +274,25 @@ nntp_folder_search_by_expression (CamelFolder *folder, const char *expression, C
 	return matches;
 }
 
+static guint32
+nntp_folder_count_by_expression (CamelFolder *folder, const char *expression, CamelException *ex)
+{
+	CamelNNTPFolder *nntp_folder = CAMEL_NNTP_FOLDER (folder);
+	guint32 count;
+	
+	CAMEL_NNTP_FOLDER_LOCK(nntp_folder, search_lock);
+	
+	if (nntp_folder->search == NULL)
+		nntp_folder->search = camel_folder_search_new ();
+	
+	camel_folder_search_set_folder (nntp_folder->search, folder);
+	count = camel_folder_search_count(nntp_folder->search, expression, ex);
+	
+	CAMEL_NNTP_FOLDER_UNLOCK(nntp_folder, search_lock);
+	
+	return count;
+}
+
 static GPtrArray *
 nntp_folder_search_by_uids (CamelFolder *folder, const char *expression, GPtrArray *uids, CamelException *ex)
 {
@@ -300,8 +319,10 @@ static void
 nntp_folder_search_free (CamelFolder *folder, GPtrArray *result)
 {
 	CamelNNTPFolder *nntp_folder = CAMEL_NNTP_FOLDER (folder);
-	
+
+	CAMEL_NNTP_FOLDER_LOCK(nntp_folder, search_lock);
 	camel_folder_search_free_result (nntp_folder->search, result);
+	CAMEL_NNTP_FOLDER_UNLOCK(nntp_folder, search_lock);
 }
 
 static void           
@@ -419,8 +440,12 @@ static void
 nntp_folder_finalise (CamelNNTPFolder *nntp_folder)
 {
 	struct _CamelNNTPFolderPrivate *p;
+
+	CamelException ex;
+
+	camel_exception_init (&ex);
 	
-	camel_folder_summary_save (((CamelFolder*) nntp_folder)->summary);
+	camel_folder_summary_save_to_db (((CamelFolder*) nntp_folder)->summary, &ex);
 	
 	p = nntp_folder->priv;
 	g_mutex_free (p->search_lock);
@@ -455,6 +480,7 @@ nntp_folder_class_init (CamelNNTPFolderClass *camel_nntp_folder_class)
 	camel_folder_class->set_message_flags = nntp_folder_set_message_flags;
 	camel_folder_class->get_message = nntp_folder_get_message;
 	camel_folder_class->search_by_expression = nntp_folder_search_by_expression;
+	camel_folder_class->count_by_expression = nntp_folder_count_by_expression;
 	camel_folder_class->search_by_uids = nntp_folder_search_by_uids;
 	camel_folder_class->search_free = nntp_folder_search_free;
 }
@@ -512,7 +538,8 @@ camel_nntp_folder_new (CamelStore *parent, const char *folder_name, CamelExcepti
 	root = g_strdup_printf("%s.ev-summary", nntp_folder->storage_path);
 	folder->summary = (CamelFolderSummary *) camel_nntp_summary_new (folder, root);
 	g_free(root);
-	camel_folder_summary_load (folder->summary);
+
+	camel_folder_summary_load_from_db (folder->summary, ex);
 	
 	si = camel_store_summary_path ((CamelStoreSummary *) ((CamelNNTPStore*) parent)->summary, folder_name);
 	if (si) {
